/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012, 2013  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.framework;

import java.awt.image.BufferedImage;



/**
 * This class contains static methods suitable for creating various procedural
 * images which is used by (for example) all BitBlt tests.
 * 
 * @author Pavel Tisnovsky
 */
public class ImageFactory
{
    /**
     * Converts boolean value into an integer representing black or white color.
     * Alpha channel is also computed correctly.
     * 
     * @param whiteColorFlag
     *            true if white color is required, false otherwise
     * @return integer representing black or white color
     */
    private static int booleanToBlackOrWhiteColor(boolean whiteColorFlag)
    {
        return whiteColorFlag ? 0xffffffff: 0xff000000;
    }

    /**
     * Add small simple label to the top left corner of the test image.
     * 
     * @param image
     *            buffered image containing test pattern
     */
    private static void drawLabelToImage(BufferedImage image)
    {
        // Array of string containing raster logo
        // ' ' == black pixel
        // '*' == white pixel
        final String[] quickAndDirtyBitmap = new String[]
        {
              "*****  ***  ****",
              "  *   *   * *   *",
              "  *   *   * ****",
              "  *   *   * *",
              "  *    ***  *",
              "",
              "",
              "  ***** *****",
              "  *     *",
              "* ***   *** *****",
              "* *     *     *",
              "* ***** *     *",
              "*             *",
              "*****         *"
        };
        // vertical position of the first row of pixels
        int y = 2;
        // render all raster lines
        for (String line : quickAndDirtyBitmap)
        {
            // horizontal position of the first column of pixels
            int x = 2;
            // render all pixels on raster line
            for (int i = 0; i < line.length(); i++)
            {
                // get pixel value
                boolean pixel = line.charAt(i) == '*';
                // convert it to RGB color
                int color = booleanToBlackOrWhiteColor(pixel);
                // and draw the pixel
                image.setRGB(x, y, color);
                x++;
            }
            y++;
        }
    }

    /**
     * Create image containing basic checker black and white pattern.
     * 
     * @param gridSize
     *            size of a grid
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createCheckerImage(int gridSize, int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // used by test for a tile colors
        int doubledGridSize = gridSize << 1;

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each tile
                boolean evenColumn = (x % doubledGridSize) >= gridSize;
                boolean evenRow = (y % doubledGridSize) >= gridSize;
                // compute color for each pixel
                boolean whiteField = evenColumn ^ evenRow;
                // convert boolean into color code
                int color = booleanToBlackOrWhiteColor(whiteField);
                // set color for given pixel
                image.setRGB(x, y, color);
            }
        }
        // we would like to know where is top left corner of this image
        drawLabelToImage(image);

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing basic diagonal checker black and white pattern.
     * Angle of diagonal checker pattern is set to 45 degrees.
     * 
     * @param gridSize
     *            size of a grid
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createDiagonalCheckerImage(int gridSize, int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // used for computing checker texture
        int width2 = width << 1;

        // used by test for a tile colors
        int doubledGridSize = gridSize << 1;

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each tile
                boolean evenColumn = ((x + y) % doubledGridSize) >= gridSize;
                boolean evenRow = ((width2 + x - y) % doubledGridSize) >= gridSize;
                // compute color for each pixel
                boolean whiteField = evenColumn ^ evenRow;
                // convert boolean into color code
                int color = booleanToBlackOrWhiteColor(whiteField);
                // set color for given pixel
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing simple grid black and white pattern.
     * 
     * @param gridSize
     *            size of a grid
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createGridImage(int gridSize, int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // used by test for a tile colors
        int doubledGridSize = gridSize << 1;

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each tile
                boolean evenColumn = (x % doubledGridSize) == 0;
                boolean evenRow = (y % doubledGridSize) == 0;
                // compute color for each pixel
                boolean blackField = evenColumn || evenRow;
                // convert boolean into color code
                int color = booleanToBlackOrWhiteColor(!blackField);
                // set color for given pixel
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing simple diagonal grid black and white pattern.
     * Angle of diagonal grid pattern is set to 45 degrees.
     * 
     * @param gridSize
     *            size of a grid
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createDiagonalGridImage(int gridSize, int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // used by test for a tile colors
        int doubledGridSize = gridSize << 1;

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each tile
                boolean evenColumn = ((x+y) % doubledGridSize) == 0;
                boolean evenRow = ((x-y) % doubledGridSize) == 0;
                // compute color for each pixel
                boolean blackField = evenColumn || evenRow;
                // convert boolean into color code
                int color = booleanToBlackOrWhiteColor(!blackField);
                // set color for given pixel
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing horizontal black and white stripes.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalStripesImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                boolean colorFlag = (y % 2) == 0;
                // convert boolean into color code
                int color = booleanToBlackOrWhiteColor(colorFlag);
                // set color for given pixel
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing vertical black and white stripes.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalStripesImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                boolean colorFlag = (x % 2) == 0;
                // convert boolean into color code
                int color = booleanToBlackOrWhiteColor(colorFlag);
                // set color for given pixel
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing diagonal black and white stripes.
     * Angle of diagonal strip pattern is set to 45 degrees.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createDiagonalStripesImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);
    
        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                boolean colorFlag = (x+y) % 2 == 0;
                // convert boolean into color code
                int color = booleanToBlackOrWhiteColor(colorFlag);
                // set color for given pixel
                image.setRGB(x, y, color);
            }
        }
    
        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing horizontal color stripes.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalColorStripesImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int color = ColorPalette.getColorAsInt(y);
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // set color for given pixel
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing vertical color stripes.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalColorStripesImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int color = ColorPalette.getColorAsInt(x);
                // set color for given pixel
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing diagonal color stripes.
     * Angle of diagonal strip pattern is set to 45 degrees.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createDiagonalColorStripesImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int color = ColorPalette.getColorAsInt(x + y);
                // set color for given pixel
                image.setRGB(x, y, color);
                // compute color for each pixel
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing black and white dots pattern.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createBWDotsPatternImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                boolean colorFlag = (x % 2) == 0 || (y % 2) == 0;
                // convert boolean into color code
                int color = booleanToBlackOrWhiteColor(colorFlag);
                // set color for given pixel
                image.setRGB(x, y, color);
                // compute color for each pixel
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing color dots pattern.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createColorDotsPatternImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                int color;
                // set color to white on even row and column 
                if (x % 2 == 0 || y % 2 == 0)
                {
                    color = -1;
                }
                else
                {
                    int red = x % 4 == 1 ? 0x00 : 0xff;
                    int green = y % 4 == 1 ? 0x00 : 0xff;
                    int blue = 0;
                    color = (0xff << 24) | (red << 16) | (green << 8) | blue;
                }
                image.setRGB(x, y, color);
                // compute color for each pixel
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing grayscale gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalGrayscaleGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            int color = (0xff << 24) | (rgb << 16) | (rgb << 8) | rgb;
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing grayscale gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalGrayscaleGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                int color = (0xff << 24) | (rgb << 16) | (rgb << 8) | rgb;
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing red gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalRedGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            int color = (0xff << 24) | (rgb << 16);
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing red gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalRedGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                int color = (0xff << 24) | (rgb << 16);
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing green gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalGreenGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            int color = (0xff << 24) | (rgb << 8);
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing green gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalGreenGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                int color = (0xff << 24) | (rgb << 8);
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing blue gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalBlueGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            int color = (0xff << 24) | (rgb);
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing blue gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalBlueGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                int color = (0xff << 24) | (rgb);
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing magenta gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalMagentaGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            int color = (0xff << 24) | (rgb << 16) | (rgb);
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing magenta gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalMagentaGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                int color = (0xff << 24) | (rgb << 16) | (rgb);
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing cyan gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalCyanGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            int color = (0xff << 24) | (rgb << 8) | (rgb);
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing cyan gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalCyanGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                int color = (0xff << 24) | (rgb << 8) | (rgb);
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing yellow gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalYellowGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            int color = (0xff << 24) | (rgb << 16) | (rgb << 8);
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing yellow gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalYellowGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                int color = (0xff << 24) | (rgb << 16) | (rgb << 8);
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing red to blue gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalRedToBlueGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            int invrgb = 255 - rgb;
            //           alpha         red  (green)  blue
            int color = (0xff << 24) | (rgb << 16) | invrgb;
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing red to blue gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalRedToBlueGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                int invrgb = 255 - rgb;
                //           alpha         red  (green)  blue
                int color = (0xff << 24) | (rgb << 16) | invrgb;
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing red to green gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalRedToGreenGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            int invrgb = 255 - rgb;
            //           alpha         red            green       blue
            int color = (0xff << 24) | (rgb << 16) | (invrgb << 8);
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing red to green gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalRedToGreenGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                int invrgb = 255 - rgb;
                //           alpha         red            green       blue
                int color = (0xff << 24) | (rgb << 16) | (invrgb << 8);
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing red to white gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createHorizontalRedToWhiteGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // compute color for each pixel
            int rgb = (int) Math.round(y * 256.0 / height);
            if (rgb < 0) rgb = 0;
            if (rgb > 255) rgb = 255;
            //           alpha         red            green       blue
            int color = (0xff << 24) | (0xff << 16) | (rgb << 8) | rgb;
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

    /**
     * Create image containing red to white gradient.
     * 
     * @param width
     *            width of a buffered image
     * @param height
     *            height of a buffered image
     * @param imageType
     *            buffered image type
     * @return buffered image containing checker pattern
     */
    public static BufferedImage createVerticalRedToWhiteGradientImage(int width, int height, int imageType)
    {
        // check for proper image dimensions
        if (width <= 0 || height <= 0)
        {
            return null;
        }

        // create new image
        BufferedImage image = new BufferedImage(width, height, imageType);

        // for all lines in a raster image
        for (int y = 0; y < height; y++)
        {
            // for all columns on a line
            for (int x = 0; x < width; x++)
            {
                // compute color for each pixel
                int rgb = (int) Math.round(x * 256.0 / width);
                if (rgb < 0) rgb = 0;
                if (rgb > 255) rgb = 255;
                //           alpha         red            green       blue
                int color = (0xff << 24) | (0xff << 16) | (rgb << 8) | rgb;
                image.setRGB(x, y, color);
            }
        }

        // return buffered image containing computed color pattern
        return image;
    }

}
