/*
  Java gfx-test framework

   Copyright (C) 2010, 2011  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.CubicCurve2D;
import org.gfxtest.framework.*;
import org.gfxtest.framework.annotations.*;



/**
 * This test renders various cubic curves using identity transformation matrix.
 * Cubic curves are constructed using CubicCurve2D class. Curves are drawn using
 * various colors and stroke width.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@GraphicsPrimitive(GraphicsPrimitives.SHAPE)
@ShapeVariant(ShapeVariants.CUBIC_CURVE)
@RenderStyle(RenderStyles.NORMAL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class NormalCubicCurves extends GfxTest
{
    /**
     * Default Y offset of curve end points.
     */
    private static final int DEFAULT_Y_OFFSET = 40;

    /**
     * Step between curves drawn by different colors onto the same image.
     */
    private static final int OFFSET_STEP = 20;

    /**
     * Draw crosses at the both end points and both control points of cubic
     * curve.
     * 
     * @param graphics
     *            graphics canvas
     * @param x
     *            array containing x coordinates of all four control points of
     *            cubic curve
     * @param y
     *            array containing y coordinates of all four control points of
     *            cubic curve
     * @param pointIndexes 
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     */
    private void drawCrossesAtControlPoints(Graphics2D graphics, int[] x, int[] y, int[] pointIndexes)
    {
        for (int i = 0; i < 4; i++)
        {
            drawCross(graphics, x[pointIndexes[i]-1], y[pointIndexes[i]-1]);
        }
    }

    /**
     * Draw cubic curve onto canvas specified by Graphics2D class.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     * @param cubicCurve
     *            cubic curve to be drawn
     * @param color
     *            curve color or null when Color.BLACK can be used.
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @return test result status - PASSED, FAILED or ERROR
     */
    private TestResult drawCubicCurve(TestImage image, Graphics2D graphics, CubicCurve2D cubicCurve, Color color, int[] pointIndexes)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(image);

        // fill in arrays containing coordinates for all cubic curve control points
        int[] x = pointSet.getXPointArray();
        int[] y = pointSet.getYPointArray();

        // construct CubicCurve2D.Float with set coordinates
        cubicCurve.setCurve(x[pointIndexes[0]-1], y[pointIndexes[0]-1],
                            x[pointIndexes[1]-1], y[pointIndexes[1]-1],
                            x[pointIndexes[2]-1], y[pointIndexes[2]-1],
                            x[pointIndexes[3]-1], y[pointIndexes[3]-1]);

        // set the specified color
        graphics.setColor(color == null ? Color.BLACK : color);

        // draw CubicCurve2D
        graphics.draw(cubicCurve);

        // Draw crosses at the both end points and both control points of cubic
        // curve.
        drawCrossesAtControlPoints(graphics, x, y, pointIndexes);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Draw cubic curve onto canvas specified by Graphics2D class.
     * Curve is drawn using black color.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     * @param cubicCurve
     *            cubic curve to be drawn
     * @param pointIndexes
     *            indexes of control point to use to draw the curve (default
     *            value should be {1,2,3,4})
     * @return test result status - PASSED, FAILED or ERROR
     */
    private TestResult drawCubicCurve(TestImage image, Graphics2D graphics, CubicCurve2D cubicCurve)
    {
        return drawCubicCurve(image, graphics, cubicCurve, Color.BLACK, new int[]{1,2,3,4});
    }

    /**
     * Draw cubic curve onto canvas specified by Graphics2D class.
     * Curve is drawn using black color.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     * @param cubicCurve
     *            cubic curve to be drawn
     * @return test result status - PASSED, FAILED or ERROR
     */
    private TestResult drawCubicCurve(TestImage image, Graphics2D graphics, CubicCurve2D cubicCurve, int[] pointIndexes)
    {
        return drawCubicCurve(image, graphics, cubicCurve, Color.BLACK, pointIndexes);
    }

    /**
     * Draw thick cubic curve onto canvas specified by Graphics2D class.
     * Curve is drawn using black color and 10 pixels wide stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     * @param cubicCurve
     *            cubic curve to be drawn
     * @return test result status - PASSED, FAILED or ERROR
     */
    private TestResult drawThickCubicCurve(TestImage image, Graphics2D graphics, CubicCurve2D cubicCurve, int[] pointIndexes)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics);

        return drawCubicCurve(image, graphics, cubicCurve, Color.BLACK, pointIndexes);
    }

    /**
     * Draw set of cubic curves onto canvas specified by Graphics2D class.
     * Curves are drawn using various colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     */
    private void drawCurveWithVariousColors(TestImage image, Graphics2D graphics)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(image);

        // curve colors
        Color[] colors = ColorPalette.getColors();

        // offset counter
        int offset = -DEFAULT_Y_OFFSET >> 1;

        // Draw each curve with different color.
        for (Color color : colors)
        {
            // create new CubicCurve2D.Float
            CubicCurve2D cubicCurve = new CubicCurve2D.Float();

            // fill in arrays containing coordinates for all cubic curve control points
            int[] x = pointSet.getXPointArray();
            int[] y = pointSet.getYPointArray();

            // construct CubicCurve2D.Float with set coordinates
            cubicCurve.setCurve(x[0], y[0] + offset, x[1], y[1] + offset, x[2], y[2] + offset, x[3], y[3] + offset);

            // set the specified color
            graphics.setColor(color);

            // draw CubicCurve2D
            graphics.draw(cubicCurve);

            // move next curve down
            offset += OFFSET_STEP;
        }
    }

    /**
     * Draw set of cubic curves onto canvas specified by Graphics2D class.
     * Curves are drawn using various transparency.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics canvas
     */
    private void drawCurveWithVariousTransparency(TestImage image, Graphics2D graphics)
    {
        // construct point set which consists of all four curve control points
        CubicCurvePointSet pointSet = new CubicCurvePointSet(image);

        // offset counter
        int offset = -DEFAULT_Y_OFFSET >> 1;

        // Draw each curve with transparency
        for (int transparency = 0; transparency < 16; transparency++)
        {
            // create new CubicCurve2D.Float
            CubicCurve2D cubicCurve = new CubicCurve2D.Float();

            // fill in arrays containing coordinates for all cubic curve control points
            int[] x = pointSet.getXPointArray();
            int[] y = pointSet.getYPointArray();

            // construct CubicCurve2D.Float with set coordinates
            cubicCurve.setCurve(x[0], y[0] + offset, x[1], y[1] + offset, x[2], y[2] + offset, x[3], y[3] + offset);

            // set the specified color
            graphics.setColor(new Color(0f, 0f, 0f, transparency / 16f));

            // draw CubicCurve2D
            graphics.draw(cubicCurve);

            // move next curve down
            offset += OFFSET_STEP;
        }
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBasicDrawCubicCurveFloat(TestImage image, Graphics2D graphics)
    {
        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Double() is rendered
     * correctly.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBasicDrawCubicCurveDouble(TestImage image, Graphics2D graphics)
    {
        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Double();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with zero pixels wide
     * stroke and default caps.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testZeroStroke(TestImage image, Graphics2D graphics)
    {
        // set zero pixels wide stroke
        CommonRenderingStyles.setStrokeZeroThick(graphics);

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 10 pixels wide
     * stroke and default caps.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testThickStroke(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics);

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 10 pixels wide
     * stroke and caps sets to CAP_BUTT.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testThickStrokeCapsButt(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 10 pixels wide
     * stroke and caps sets to CAP_ROUND.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testThickStrokeCapsRound(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 10 pixels wide
     * stroke and caps sets to CAP_SQUARE.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testThickStrokeCapsSquare(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 30 pixels wide
     * stroke and default caps.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testExtraThickStroke(TestImage image, Graphics2D graphics)
    {
        // set 30 pixels wide stroke
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics);

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 30 pixels wide
     * stroke and caps sets to CAP_BUTT.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testExtraThickStrokeCapsButt(TestImage image, Graphics2D graphics)
    {
        // set 30 pixels wide stroke
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 30 pixels wide
     * stroke and caps sets to CAP_ROUND.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testExtraThickStrokeCapsRound(TestImage image, Graphics2D graphics)
    {
        // set 30 pixels wide stroke
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is to be drawn with 30 pixels wide
     * stroke and caps sets to CAP_SQUARE.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testExtraThickStrokeCapsSquare(TestImage image, Graphics2D graphics)
    {
        // set 30 pixels wide stroke
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve
        return drawCubicCurve(image, graphics, cubicCurve);
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColors(TestImage image, Graphics2D graphics)
    {
        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors and using 10 pixel wide
     * stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsStrokeWidth(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics); 

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered correctly.
     * Curve is drawn using different colors and using zero pixel wide stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsZeroStrokeWidth(TestImage image, Graphics2D graphics)
    {
        // set zero pixels wide stroke
        CommonRenderingStyles.setStrokeZeroThick(graphics); 

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors and using 10 pixel wide
     * stroke using CAP_BUTT.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsCapsButt(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors and using 10 pixel wide
     * stroke using CAP_ROUND.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsCapsRound(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_ROUND, BasicStroke.JOIN_BEVEL);

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered
     * correctly. Curve is drawn using different colors and using 10 pixel wide
     * stroke using CAP_SQUARE.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testColorsCapsSquare(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL);

        // draw set of curves using test color palette
        drawCurveWithVariousColors(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered correctly.
     * Curve is drawn using different transparency and using 1 pixel wide
     * stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testTransparencyBasicStroke(TestImage image, Graphics2D graphics)
    {
        // draw set of curves using different transparency
        drawCurveWithVariousTransparency(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered correctly.
     * Curve is drawn using different transparency and using 10 pixel wide
     * stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testTransparencyThickStroke(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeThickWidth(graphics);

        // draw set of curves using different transparency
        drawCurveWithVariousTransparency(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered correctly.
     * Curve is drawn using different transparency and using 30 pixel wide
     * stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testTransparencyExtraThickStroke(TestImage image, Graphics2D graphics)
    {
        // set 10 pixels wide stroke
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics);

        // draw set of curves using different transparency
        drawCurveWithVariousTransparency(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test if cubic curve created by CubicCurve2D.Float() is rendered correctly.
     * Curve is drawn using different transparency and using 30 pixel wide
     * stroke.
     * 
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testTransparencyZeroThickStroke(TestImage image, Graphics2D graphics)
    {
        // set zero pixels wide stroke
        CommonRenderingStyles.setStrokeZeroThick(graphics);

        // draw set of curves using different transparency
        drawCurveWithVariousTransparency(image, graphics);

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test drawing cubic curve with no identical points.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testNoIdenticalPoints(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 2, 3, 4 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 1 and 2.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints12(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 1, 3, 4 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 1 and 3.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints13(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 2, 1, 4 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 1 and 4.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints14(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 2, 3, 1 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 2 and 3.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints23(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 2, 2, 4 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 2 and 4.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints24(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 2, 3, 2 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 3 and 4.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints34(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 2, 3, 3 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 1, 2 and 3.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints123(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 1, 1, 4 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 1, 2 and 4.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints124(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 1, 3, 1 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 1, 3 and 4.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints134(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 2, 1, 1 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 2, 3 and 4.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints234(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 2, 2, 2 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Test special test case - cubic curve with identical points 1, 2, 3 and 4.
     * To be precise: first point and fourth point are curve end points and
     * second point and third point are control points which defines the curvature.
     *
     * @param image
     *            image to which two dimensional shape is to be rendered
     * @param graphics
     *            graphics context for image
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIdenticalPoints1234(TestImage image, Graphics2D graphics)
    {
        // control point indexes
        int[] controlPointIndexes = new int[]
        { 1, 1, 1, 1 };

        // create new CubicCurve2D.Float
        CubicCurve2D cubicCurve = new CubicCurve2D.Float();

        // draw cubic curve and return test result
        return drawThickCubicCurve(image, graphics, cubicCurve, controlPointIndexes);
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new NormalCubicCurves().runTestSuite(args);
    }
}
